/***************************************************************************
                          bhv_player.c  -  description
                             -------------------
    begin                : Sat Apr 5 2003
    copyright            : (C) 2003 by Steven Goodwin
    email                : 
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include <malloc.h>
#include <string.h>
#include "explore.h"
#include "level.h"
#include "render.h"
#include "objects.h"
#include "collision.h"

#define K_PLY_INVINCIBILITY_TIMEOUT		50

#define K_PLAYERFALLJUMPSPEED			2
#define K_PLAYERCRAWLSPEED			3
#define K_PLAYERWALKSPEED			4
#define K_PLAYERJUMPSPEED			5

#define K_MOVE_NORMAL				0
#define K_MOVE_FALLING				1
#define K_MOVE_SLIDE_LEFT			2
#define K_MOVE_SLIDE_RIGHT			4	
#define K_MOVE_JUMPING				8

typedef enum  {
	eWalking,
	eJumping,
	eFalling,
	eDead,
	}  tPlyState;

typedef enum  {
	eNoCollision,
	eCollision,
	eSlideOccured,
	eJumperHitHead,
	}  tPlyCollisionCheck;

typedef struct sPLY_STATE {
	int		iLives;
	int		iAirSupply;
	int		iScore;
	int		iKeysLeft;

	tPlyState	iState;

	int		iAnimFrame;
	int		iAnimOffset;

	int		iJumpDirection;
	int		iJumpYBase;
	int		iJumpFrame;
	BOOL		bJumpUpPhase;

	int		iLastSolidFloor;
	int		iFallDirection;
	int		iRespawnInvincibility;
	BOOL		bIsPlayerInvincible;

	BOOL		bIsWalking;
	int		iCrouchWalkTransition;

	BOOL		bAllKeysReleased;		/* for death */
	} EX_PLY_STATE;

static void plyStart(EX_OBJECT *pObj);
static void plyReset(EX_OBJECT *pObj);
static void plyUpdate(EX_OBJECT *pObj, int frames);
static tObjType plyGetType(const EX_OBJECT *pObj);
static void plyDraw(const EX_OBJECT *pObj);
static void plyDestroy(EX_OBJECT *pObj);

static void plyChangeToState(const EX_OBJECT *pObj, tPlyState new_state, int iParam);


BOOL Bhv_Player(EX_OBJECT *pObj, const char *pParams)
{
	pObj->pGfx = TheGame.pPlayerGfx;
	pObj->VTable.Start = plyStart;
	pObj->VTable.Reset = plyReset;
	pObj->VTable.GetType = plyGetType;
	pObj->VTable.Update = plyUpdate;
	pObj->VTable.Draw = plyDraw;
	pObj->VTable.Destroy = plyDestroy;

	/* Parse params into our own private setup structure */
	pObj->pBhvStateData = malloc(sizeof(EX_PLY_STATE));
	if (!pObj->pBhvStateData)
		return FALSE;

return TRUE;
}

static void plyStart(EX_OBJECT *pObj)
{
EX_PLY_STATE *pState = pObj->pBhvStateData;

	plyReset(pObj);
	pState->iLives = 5;
	pState->iScore = 0;
}

static void plyReset(EX_OBJECT *pObj)
{
EX_PLY_STATE *pState = pObj->pBhvStateData;
EX_OBJECT *pObjList = TheGame.curr_level.pObjList;

	pObj->x = pObj->init_x;
	pObj->y = pObj->init_y;
	pState->iAirSupply = 1280;
	pState->iState = eWalking;
	pState->iAnimFrame = 0;
	pState->iAnimOffset = T_PLAYER_WALK_RIGHT;
	pState->iRespawnInvincibility = 0;
	pState->bIsPlayerInvincible = TRUE;
	pState->iCrouchWalkTransition = 0;
	pState->bIsWalking = TRUE;

	pState->iKeysLeft = 0;
	while(pObjList)
		{
		if (exGetTypeObject(pObjList) == eKey)
			{
			pState->iKeysLeft++;
			}

		pObjList=pObjList->pNext;
		}

}

static tObjType plyGetType(const EX_OBJECT *pObj)
{
	return ePlayer;
}

static void plyDestroy(EX_OBJECT *pObj)
{
EX_PLY_STATE *pState = pObj->pBhvStateData;

	free(pState);
}

/*
** Players state machine
*/
void plyGetPlayerRectAt(const EX_OBJECT *pPlyObj, int x, int y, SDL_Rect *pRc)
{
EX_PLY_STATE *pState = pPlyObj->pBhvStateData;

	/* Prepare the players collision to a smaller box, reflecting the character */
	pRc->x = x+TheGame.iTileWidth/4;
	pRc->y = y;
	pRc->w = TheGame.iTileWidth/2;
	pRc->h = TheGame.iTileHeight;
	if (!pState->bIsWalking)
		{
		pRc->y += TheGame.iTileHeight/2;
		pRc->h -= TheGame.iTileHeight/2;
		}
}

void plyGetPlayerRect(const EX_OBJECT *pPlyObj, SDL_Rect *pRc)
{
	plyGetPlayerRectAt(pPlyObj, pPlyObj->x, pPlyObj->y, pRc);
}


#define ABS(__v)   ((__v)<0?-(__v):(__v))

tPlyCollisionCheck plyCheckCollisions(const EX_OBJECT *pPlyObj, int oldx, int oldy, int newx, int newy, int *pResultX, int *pResultY, int iSpecial)
{
/* Check every position from old to new. Return the position of the last safe place */
EX_COLLISION_LIST clist;
SDL_Rect rc;
int steps, count;
int testx, testy;
int safex, safey;
BOOL bIgnoreXComponent = FALSE;
tPlyCollisionCheck rt = eNoCollision;

	steps = ABS(newx-oldx);
	if (ABS(newy-oldy) > steps)
		steps = ABS(newy-oldy);

	safex = oldx;
	safey = oldy;
	for(count=1;count<=steps;count++)
		{
		if (!bIgnoreXComponent)	/* otherwise keep old testx */
			testx = oldx + ((newx-oldx)*count)/steps;

		testy = oldy + ((newy-oldy)*count)/steps;

		plyGetPlayerRectAt(pPlyObj, testx, testy, &rc);

		exInitCollisionList(&clist);
		clist.bCheckObjects[ePlayer] = FALSE;
		clist.bCheckObjects[eKey] = FALSE;
		clist.bCheckObjects[eDeadly] = FALSE;
		clist.bCheckObjects[eExitGate] = FALSE;

		exGetRectCollision(pPlyObj, &rc, &clist);

		/* Try moving without X direction component */
		if (clist.iSize && (iSpecial & (K_MOVE_FALLING | K_MOVE_JUMPING)))
			{
			testx = safex;
			clist.iSize = 0;
			plyGetPlayerRectAt(pPlyObj, testx, testy, &rc);
			exGetRectCollision(pPlyObj, &rc, &clist);
			/* if that works, ignore the X movement */
			if (clist.iSize == 0)
				{
				bIgnoreXComponent = TRUE;
				}
			else
				{
				rt = (iSpecial & K_MOVE_FALLING)?eCollision:eJumperHitHead;
				break;
				}
			}

		if (clist.iSize)
			{
			rt = eCollision;
			break;	/* exit loop */
			}

		if (iSpecial & (K_MOVE_SLIDE_RIGHT | K_MOVE_SLIDE_LEFT))
			{
			if ((testy % TheGame.iTileHeight)==0)
				{
				EX_TILE *pTile;
				int tx,ty, ttx;

				/* Find tile to left/right of us */
				tx = testx / TheGame.iTileWidth;
				ty = testy / TheGame.iTileHeight;

				if (iSpecial & K_MOVE_SLIDE_RIGHT)
					tx++;
				else if ((testx%TheGame.iTileWidth)==0)
					tx--;
		
				pTile = exGetTileAt(tx, ty);
				if (pTile && pTile->iRegion == 0)
					{
					/* if there's a gap; check that there's a ledge beneath us */
					pTile = exGetTileAt(tx, ty+1);
					if (pTile && pTile->iRegion != 0)
						{
						ttx = testx + ((iSpecial & K_MOVE_SLIDE_RIGHT)?1:-1);
						plyGetPlayerRectAt(pPlyObj, ttx, testy, &rc);
						exGetRectCollision(pPlyObj, &rc, &clist);
						if (clist.iSize == 0)
							{
							safex = ttx;
							safey = testy;
							rt = eSlideOccured;
							break;
							}
						}
					}
				}
			}

		/* No collisions! Mark this spot as the new safe spot. */
		safex = testx;
		safey = testy;
		}

	*pResultX = safex;
	*pResultY = safey;

return rt;
}

BOOL plyCanIStandUp(EX_OBJECT *pPlyObj)
{
EX_PLY_STATE *pState = pPlyObj->pBhvStateData;
SDL_Rect rc;
EX_COLLISION_LIST collisions;

	if (pState->bIsWalking)
		return TRUE;

	plyGetPlayerRect(pPlyObj, &rc);	/* the crouched volume */

	rc.y -= TheGame.iTileHeight/2;
	rc.h += TheGame.iTileHeight/2;

	exInitCollisionList(&collisions);
	if (exGetRectCollision(pPlyObj, &rc, &collisions))
		return FALSE;

return TRUE;
}

static void plyUpdWalking(EX_OBJECT *pObj, EX_PLY_STATE *pState, int frames)
{
int newx, newy;
int speed;
SDL_Rect rc;
EX_COLLISION_LIST collisions;
EX_OBJECT *pCollisionObj;

	newx = pObj->x;
	newy = pObj->y;

	speed = (pState->bIsWalking)? K_PLAYERWALKSPEED : K_PLAYERCRAWLSPEED;

	if (pState->iCrouchWalkTransition == 1)
		{
		pState->bIsWalking = !pState->bIsWalking;

		if (pState->bIsWalking)
			pState->iAnimOffset = T_PLAYER_WALK_LEFT + pState->iAnimOffset%20;
		else
			pState->iAnimOffset = T_PLAYER_CROUCH_LEFT + pState->iAnimOffset%20;

		pState->iCrouchWalkTransition = 2;
		}

	if (TheGame.iface.iMove[EXI_MOVE_LEFT])
		{
		newx -= speed * frames;
		if (pState->bIsWalking)
			pState->iAnimOffset = T_PLAYER_WALK_LEFT;
		else
			pState->iAnimOffset = T_PLAYER_CROUCH_LEFT;
		}

	if (TheGame.iface.iMove[EXI_MOVE_RIGHT])
		{
		newx += speed * frames;
		if (pState->bIsWalking)
			pState->iAnimOffset = T_PLAYER_WALK_RIGHT;
		else
			pState->iAnimOffset = T_PLAYER_CROUCH_RIGHT;
		}

	if (newx != pObj->x)
		{
		plyCheckCollisions(pObj, pObj->x, pObj->y, newx, newy, &newx, &newy, K_MOVE_NORMAL);

		/* We're still walking - so update our position and continue */
		if (pObj->x != newx)
			{
			if (++pState->iAnimFrame == 8)
				pState->iAnimFrame = 0;
			}

		pObj->x = newx;
		pObj->y = newy;
		}

	/* Are we still on a floor? Should we fall? */
	plyGetPlayerRect(pObj, &rc);

	/* Now move the position to look at the area just below the feet */
	rc.y += rc.h;
	rc.h = 1;
	exInitCollisionList(&collisions);
	if (exGetRectCollision(pObj, &rc, &collisions))
		{
		pCollisionObj = collisions.List[0].pObj;
		if (pCollisionObj && exGetTypeObject(pCollisionObj) == eFallingFloor)
			{
			ffDistintigrateFloor(pCollisionObj, frames);
			}
		}
	else
		{
		plyChangeToState(pObj, eFalling, 0);
		return;
		}

	/* Are we trying to jump? (if we reach here, we must be on solid ground,
	   so we may jump) */
	if (TheGame.iface.iMove[EXI_MOVE_JUMP] && pState->bIsWalking)
		{
		plyGetPlayerRectAt(pObj, newx, newy-TheGame.iTileHeight, &rc);

		exInitCollisionList(&collisions);
		collisions.bCheckObjects[ePlayer] = FALSE;
		collisions.bCheckObjects[eKey] = FALSE;
		collisions.bCheckObjects[eDeadly] = FALSE;
		collisions.bCheckObjects[eExitGate] = FALSE;

		exGetRectCollision(pObj, &rc, &collisions);

		if (collisions.iSize == 0)
			{
			plyChangeToState(pObj, eJumping, 0);
			}
		}

	if (TheGame.iface.iMove[EXI_MOVE_DOWN] && pState->iCrouchWalkTransition == 0)
		{
		if (plyCanIStandUp(pObj))
			{
			pState->iCrouchWalkTransition = 1;

			/* This can be done using basic maths - but it makes reading the */
			/* code more difficult, so I won't do it! */
			if (pState->iAnimOffset == T_PLAYER_WALK_LEFT || pState->iAnimOffset == T_PLAYER_CROUCH_LEFT)
				{                
				pState->iAnimOffset = T_PLAYER_CROUCH2WALK_LEFT;
				}
			else
				{
				pState->iAnimOffset = T_PLAYER_CROUCH2WALK_RIGHT;
				}
			}
		}

	if (!TheGame.iface.iMove[EXI_MOVE_DOWN])
		pState->iCrouchWalkTransition = 0;

	pState->iLastSolidFloor = pObj->y;
}

static void plyUpdJumping(EX_OBJECT *pObj, EX_PLY_STATE *pState, int frames)
{
static int iJumpOffset[] = {0,8,18,33,44,60,65,70,72,73,};
static int iMaxJumpFrames = sizeof(iJumpOffset)/sizeof(iJumpOffset[0]);
int wantedx, wantedy;
int newx, newy;
int iSpecial;
tPlyCollisionCheck col;

	/* Move */
	wantedx = pObj->x + frames * pState->iJumpDirection * K_PLAYERJUMPSPEED;
	wantedy = (pState->iJumpYBase - iJumpOffset[pState->iJumpFrame]);

	iSpecial = K_MOVE_JUMPING;
	if (pState->iJumpDirection < 0)
		iSpecial |= K_MOVE_SLIDE_LEFT;
	else if (pState->iJumpDirection > 0)
		iSpecial |= K_MOVE_SLIDE_RIGHT;

	col = plyCheckCollisions(pObj, pObj->x, pObj->y, wantedx, wantedy, &newx, &newy, iSpecial);

	pObj->x = newx;
	pObj->y = newy;
	if (col == eSlideOccured || col == eJumperHitHead || col == eCollision)
		plyChangeToState(pObj, eWalking, 0);

	/* End of jump anim... */
	if (pState->bJumpUpPhase)
		{
		if (++pState->iJumpFrame == iMaxJumpFrames)
			{
			pState->iJumpFrame--;
			pState->bJumpUpPhase = FALSE;
			}
		}
	else /* on the way down.. */
		{
		if (pState->iJumpFrame-- == 0)
			{
			plyChangeToState(pObj, eFalling, pState->iJumpDirection * K_PLAYERFALLJUMPSPEED);
			}
		}
}

static void plyUpdFalling(EX_OBJECT *pObj, EX_PLY_STATE *pState, int frames)
{
const int iFakeGravity = 9;
int newx, newy;
int iSpecial = K_MOVE_FALLING;

	newx = pObj->x + frames * pState->iFallDirection;
	newy = pObj->y + iFakeGravity;

	if (pState->iFallDirection < 0)
		iSpecial |= K_MOVE_SLIDE_LEFT;
	else if (pState->iFallDirection > 0)
		iSpecial |= K_MOVE_SLIDE_RIGHT;

	if (eCollision == plyCheckCollisions(pObj, pObj->x, pObj->y, newx, newy, &pObj->x, &pObj->y, iSpecial))
		{
		if (pObj->y - pState->iLastSolidFloor > 6*32)
			{
			plyChangeToState(pObj, eDead, 0);
			}
		else
			{
			plyChangeToState(pObj, eWalking, 0);
			}
		}
}

static void plyUpdDead(EX_OBJECT *pObj, EX_PLY_STATE *pState, int frames)
{
	pState->iAnimFrame += frames;
	if (pState->iAnimFrame >= T_PLAYER_DEATH_END)
		pState->iAnimFrame = T_PLAYER_DEATH_END;

	/* A kind of 'press any key' thing */
	if (TheGame.iface.iMove[EXI_MOVE_JUMP] ||
		TheGame.iface.iMove[EXI_MOVE_LEFT] ||
		TheGame.iface.iMove[EXI_MOVE_RIGHT])
		{
		if (!pState->bAllKeysReleased)
			{}
		else if (pState->iLives)
			{
			exResetLevel();
			}
		else
			{
			TheGame.bIsRunning = FALSE;
			}
		}
	else
		{
		pState->bAllKeysReleased = TRUE;
		}
}

static void plyChangeToState(const EX_OBJECT *pObj, tPlyState new_state, int iParam)
{
EX_PLY_STATE *pState = pObj->pBhvStateData;

	switch(new_state)
		{
		case	eWalking:
			 	break;

		case	eJumping:
				pState->iJumpFrame = 1; /* the first frame as dy=0 */
				pState->bJumpUpPhase = TRUE;
				pState->iJumpYBase = pObj->y;
				if (TheGame.iface.iMove[EXI_MOVE_LEFT])
					pState->iJumpDirection = -1;
				else if (TheGame.iface.iMove[EXI_MOVE_RIGHT])
					pState->iJumpDirection = 1;
				else
					pState->iJumpDirection = 0;

				break;

		case	eFalling:
				pState->iFallDirection = iParam;
				break;

		case	eDead:	
				if (pState->iState != eDead)
					{
					pState->bAllKeysReleased = FALSE;
					pState->iLives--;
					pState->iAnimFrame = T_PLAYER_DEATH_START;
					}
				break;

		}

	pState->iState = new_state;
}

void plyCheckForKeys(EX_OBJECT *pPlyObj)
{
EX_OBJECT *pObj = TheGame.curr_level.pObjList;
EX_PLY_STATE *pState = pObj->pBhvStateData;
int iProximity = TheGame.iTileWidth/2;
int dx, dy;

	while(pObj)
		{
		if (exGetTypeObject(pObj) == eKey)
			{
			dx = pObj->x - pPlyObj->x;
			dy = pObj->y - pPlyObj->y;

			if (dx*dx + dy*dy < iProximity*iProximity)
				{
				if (keyIsAvailable(pObj))
					{
					++pState->iScore;
					keyPickup(pObj);
					if (--pState->iKeysLeft == 0)
						exitOpenExitGates();
					}
				}
			}
		pObj = pObj->pNext;
		}
}

static BOOL plyCheckForEnemies(const EX_OBJECT *pPlyObj)
{
EX_PLY_STATE *pState = pPlyObj->pBhvStateData;
EX_OBJECT *pObj, *pKillerObj;
SDL_Rect ply;
SDL_Rect nme;

	if (pState->bIsPlayerInvincible)
		return FALSE;

	if (pState->iState == eDead)
		return FALSE;

	plyGetPlayerRect(pPlyObj, &ply);

	pKillerObj = NULL;
	pObj =  TheGame.curr_level.pObjList;
	while(pObj)
		{
		if (exGetTypeObject(pObj) == eDeadly)
			{
			nme.x = pObj->x;
			nme.y = pObj->y;
			nme.w = TheGame.iTileWidth;
			nme.h = TheGame.iTileHeight;
			if (pObj->VTable.ProcessCollision)
				{
				/* Create an empty collision list, an ask the object */
				/* if it collides */
				EX_COLLISION_LIST CollisionsList;

				exInitCollisionList(&CollisionsList);
				CollisionsList.pIgnoreObj = NULL;
				CollisionsList.iType = ePixelData;
				plyGetPlayerRect(pPlyObj, &CollisionsList.rc);

				exProcessCollisionObject(pObj, &CollisionsList);
				if (CollisionsList.iSize)
					pKillerObj = pObj;
				}
			else if (exDoBoxesIntersect(&ply, &nme))
				{
				pKillerObj = pObj;
				}
			}
		pObj = pObj->pNext;
		}

	if (pKillerObj)
		{
		plyChangeToState(pPlyObj, eDead, 0);
		}

return pKillerObj?TRUE:FALSE;
}

static void plyCheckForExit(EX_OBJECT *pPlyObj)
{
EX_OBJECT *pObj = TheGame.curr_level.pObjList;
EX_PLY_STATE *pState = pObj->pBhvStateData;
SDL_Rect ply, exitbox;

	if (pState->iKeysLeft > 0)
		return;

	plyGetPlayerRect(pPlyObj, &ply);
	while(pObj)
		{
		if (exGetTypeObject(pObj) == eExitGate)
			{
			exitbox.x = pObj->x;
			exitbox.y = pObj->y;
			exitbox.w = TheGame.iTileWidth*2;
			exitbox.h = TheGame.iTileHeight*2;
			if (exDoBoxesIntersect(&ply, &exitbox))
				{
				TheGame.bIsRunning = FALSE;
				}
			}

		pObj=pObj->pNext;
		}
}

static void plyUpdate(EX_OBJECT *pObj, int frames)
{
EX_PLY_STATE *pState = pObj->pBhvStateData;

	switch(pState->iState)
		{
		case	eWalking:	 	plyUpdWalking(pObj, pState, frames); break;
		case	eJumping:		plyUpdJumping(pObj, pState, frames); break;
		case	eFalling:	 	plyUpdFalling(pObj, pState, frames); break;
		case	eDead:	 	plyUpdDead(pObj, pState, frames); break;
		}

	plyCheckForKeys(pObj);
	plyCheckForExit(pObj);
	plyCheckForEnemies(pObj);

	if (pState->bIsPlayerInvincible)
		{
		pState->iRespawnInvincibility += frames;
		if (pState->iRespawnInvincibility >= K_PLY_INVINCIBILITY_TIMEOUT)
			pState->bIsPlayerInvincible = FALSE;
		}

	pState->iAirSupply -= frames;
	if (pState->iAirSupply < 0)
		{
		pState->iAirSupply = 0;
		plyChangeToState(pObj, eDead, 0);
		}
}

static void plyDraw(const EX_OBJECT *pObj)
{
static int iLivesAnim[]={0,1,2,3,8,9,10,11,12,};
static int iLivesAnimIdx=0;
const int iFromAlpha = 64;
const int iToAlpha = 255;
EX_PLY_STATE *pState = pObj->pBhvStateData;
BOOL bDraw=TRUE;
int iFrame;
int i, x;

	iFrame = pState->iAnimFrame+pState->iAnimOffset;

	if (pState->iState == eDead)	
		{
		iFrame = pState->iAnimFrame;
		if (iFrame == T_PLAYER_DEATH_END)
			bDraw = FALSE;
		}

	if (pState->bIsPlayerInvincible)
		{
		Uint8 old_alpha = pObj->pGfx->format->alpha;
		SDL_SetAlpha(pObj->pGfx, SDL_SRCALPHA, iFromAlpha + ((iToAlpha-iFromAlpha)*pState->iRespawnInvincibility)/K_PLY_INVINCIBILITY_TIMEOUT);
		exDrawTile(pObj->pGfx, iFrame, pObj->x, pObj->y);
		SDL_SetAlpha(pObj->pGfx, SDL_SRCALPHA, old_alpha);
		exDrawTile(pObj->pGfx, T_PLAYER_INVINCIBLE+(4*pState->iRespawnInvincibility/K_PLY_INVINCIBILITY_TIMEOUT), pObj->x, pObj->y);
		}
      else if (bDraw)
		{
		exDrawTile(pObj->pGfx, iFrame, pObj->x, pObj->y);
		}


	/* Draw blank line at bottom for lives */
	for(i=0;i<TheGame.iNumTileWidth;i++)
		exDrawTile(TheGame.pGeneralTileGfx, 0, i*TheGame.iTileWidth, TheGame.iScreenHeight-TheGame.iTileHeight);

	/* Draw lives counter */

	/* If we had some animating characters, the lives counter would animate! */
	/* Yes - it is very 'Manic Miner'! Mathew Smith - nuff respect :) */

	for(i=0;i<pState->iLives;i++)
		exDrawTile(pObj->pGfx, iLivesAnim[iLivesAnimIdx/10], i*40+10, TheGame.iScreenHeight-TheGame.iTileHeight);
	if (++iLivesAnimIdx == 10*sizeof(iLivesAnim)/sizeof(iLivesAnim[0]))
		iLivesAnimIdx = 0;

	/* Draw Score */
	exDrawLevelTile(0,0);	/* repair background */
	exDrawLevelTile(1,0);	/* repair background */
	exWriteNumber(pState->iScore, 2, 0,0);

	/* Draw air supply */
	x = TheGame.iScreenWidth - TheGame.iTileWidth*10;

	/* Repair background */
	for(i = x/TheGame.iTileWidth; i < TheGame.iScreenWidth; ++i)
		exDrawLevelTile(i, 0);

	exDrawTile(TheGame.pGeneralTileGfx, T_AIRSUPPLY, x, 0);
	exDrawTile(TheGame.pGeneralTileGfx, T_AIRSUPPLY+1, x+TheGame.iTileWidth, 0);

	exDrawArea(TheGame.pGeneralTileGfx,
		/*src*/12*TheGame.iTileWidth, 4*TheGame.iTileHeight, pState->iAirSupply/10, TheGame.iTileHeight,
		/*dest*/x+TheGame.iTileWidth*2, 0);

}

